/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

\*---------------------------------------------------------------------------*/

#include "massFluxFvPatchVectorField.H"
#include "addToRunTimeSelectionTable.H"
#include "volFields.H"
#include "surfaceFields.H"
#include "fvcMeshPhi.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

massFluxFvPatchVectorField::massFluxFvPatchVectorField
(
    const fvPatch& p,
    const vectorField& iF
)
:
  fixedValueFvPatchVectorField(p, iF),
  mf_(p.size(),0)
{
}


massFluxFvPatchVectorField::massFluxFvPatchVectorField
(
    const massFluxFvPatchVectorField& ptf,
    const fvPatch& p,
    const vectorField& iF,
    const fvPatchFieldMapper& mapper
)
:
  fixedValueFvPatchVectorField(ptf, p, iF, mapper),
  mf_(ptf.mf_,mapper)
{  
}


massFluxFvPatchVectorField::massFluxFvPatchVectorField
(
    const fvPatch& p,
    const vectorField& iF,
    const dictionary& dict
)
:
  fixedValueFvPatchVectorField(p, iF),
  mf_("massFluxPerSquareMeter",dict,p.size())
{
    fvPatchVectorField::operator=(vectorField("value", dict, p.size()));
}


massFluxFvPatchVectorField::massFluxFvPatchVectorField
(
    const massFluxFvPatchVectorField& pivpvf,
    const vectorField& iF
)
:
  fixedValueFvPatchVectorField(pivpvf, iF),
  mf_(pivpvf.mf_)
{
}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //


void massFluxFvPatchVectorField::autoMap
(
    const fvPatchFieldMapper& m
)
{
    vectorField::autoMap(m);
    mf_.autoMap(m);
}


void massFluxFvPatchVectorField::rmap
(
    const fvPatchVectorField& ptf,
    const labelList& addr
)
{
    fixedValueFvPatchVectorField::rmap(ptf, addr);

    const massFluxFvPatchVectorField& tiptf =
        refCast<const massFluxFvPatchVectorField>(ptf);

    mf_.rmap(tiptf.mf_, addr);
}

// Update the coefficients associated with the patch field
void massFluxFvPatchVectorField::updateCoeffs()
{
    if (updated())
    {
        return;
    }

    const volScalarField& rho = db().lookupObject<volScalarField>("rho");
 
    const fvPatchField<scalar>& rhop =
        patchField<volScalarField, scalar>(rho);

    const vectorField& n = patch().nf();

//     Info << mf_;
//     Info << rhop;
//     Info << n*(mf_/rhop);

    operator==(-(n*(mf_/rhop)));

    fixedValueFvPatchVectorField::updateCoeffs();

//     Info << *this;
}


// Write
void massFluxFvPatchVectorField::write(Ostream& os) const
{
    fvPatchVectorField::write(os);
    writeEntry("value", os);
    mf_.writeEntry("massFluxPerSquareMeter",os);
}


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

makePatchTypeField(fvPatchVectorField, massFluxFvPatchVectorField);

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// ************************************************************************* //
